# Open Dataset for the CERBERO Smart Traveling use case

This dataset contains the open data for the Smart Traveling for Electric Vehicles use case.
The data in this set is intended to provide insight regarding the route planning process developed in the use case. This process encompasses the following CERBERO tools:

- DynAA for modeling the battery consumption of an Electric Vehicle for planned routes
- MECA for filtering and ranking routes planned by DynAA based on user preferences

In addition, the following tools are part of the process:

- a dedicated human-machine interface (HMI)
- the Open Source Routing Machine (OSRM)

## Contents

- OpenStreetMap data
- Charging point data
- Example communication data between MECA, the HMI, and DynAA
  - from HMI to MECA - planning constraints, origin, and destination
  - from MECA to DynAA - planning constraints
  - from MECA to DynAA - route to be evaluated
  - from DynAA to MECA - computed itineraries
  - from MECA to HMI - ranked itineraries

## OpenStreetMap geographic data

Route planning is performed using OSRM, invoked by MECA. This software employs geographical data from the OpenStreetMap (OSM) project to calculate routes. 
OSM data can be retrieved, for example, from the Geofabrik website (https://www.geofabrik.de/geofabrik/openstreetmap.html).

The region dataset employed for the Smart Traveling use case is the Italy/Nord-Ovest dataset, available from http://download.geofabrik.de/europe/italy/nord-ovest-latest.osm.pbf.

## Charging point data

Within the Smart Traveling demonstrator, planned routes will often incorporate stops at charging points. The properties of these charging points are stored in MECA's knowledge base.
The knowledge base provides the following properties of charging points for route planning:

- location (GPS coordinates)
- charging speed (W)
- charging price (euros per kWh)
- organization owning the charging point (referred to as 'brand' in the dataset)
- a list of facilities that are available at the same location, such as a coffee house, restaurant, or hotel. These are relevant for assigning scores to calculated itineraries (see [below](###-5-ranked-itinerary-response-from-meca-to-the-hmi)).

Example charging points, represented by how they are transferred to DynAA, are provided in the charging point JSON example (`charging_points.json`).

Please note that the charging points and their properties are fictional.

## Communication data

The communication data covers the full planning process performed by MECA and DynAA needed to provide a set of ranked itineraries to the HMI.

### 1: Planning request from the HMI to MECA

An itinerary planning process is initiated by the HMI by sending a 'calculation' request to MECA.
The data sent with this request contains planning constraints:

- Origin (longitude/latitude coordinates)
- Destination (longitude/latitude coordinates)
- Current battery level (percentage)
- Battery capacity (kWh)
- Minimum battery state of charge (SoC) allowed on arrival (percentage)
- Maximum number of allowed stops
- Earliest departure (date-time)
- Latest arrival (date-time)

An example planning request is provided in this file: `interface_data/hmi_meca_request.json`.

Data provided as planning constraints must adhere to the constraints schema (`interface_schemas/constraints_schema_v2.json`).

### 2: Planning constraints request from MECA to DynAA

The behavior of DynAA is controllable through a set of adjustable constraints. Typically, these are set by MECA before evaluating routes.
For the most part, the constraints forwarded to DynAA are the same as those provided to MECA by the HMI.

However, there are two differences:

- The origin and destination coordinates are not part of this request. They have been used by OSRM to generate route data.
- DynAA supports the specification of charging points reservation timeslots, during which charging points provided along routes are considered to be unavailable. When applicable, these are appended by MECA.

An example request is provided in this file: `interface_data/meca_dynaa_constraints_request.json`.

Also in this case, the data must adhere to the constraints schema (`interface_schemas/constraints_schema_v2.json`).

In response to this request, DynAA sends the full set of currently applied constraints, such that MECA can perform a sanity check.

### 3: Route evaluation request from MECA to DynAA

Requests to DynAA for route evaluation must adhere to the itinerary schema (`interface_schemas/itinerary_schema_v2.json`). This schema corresponds for the largest part to the data format in which routes are generated by OSRM, with the exception that some fields have been added.

Each request is uniquely identified using a UUID and contains a single route to be evaluated, as well as a list of waypoints. These waypoints can simply be start and stop points for the vehicle, but also include charging points where the vehicle can charge its battery. These charging points can also include information about the brand providing it, the charging speed, charging price and a list of available facilities.

The route is composed of the total travel distance and travel time, the (map) geometry of the route (specified as a polyline string), and the separate route legs of which the route is comprised. These route legs contain their individual distance and travel time as well as a list of travel speed, distance and time for each segment of the route leg. The route legs can also contain information on the inclination and composition of the road (e.g. asphalt or dirt).

An example evaluation request is provided in this file: `interface_data/meca_dynaa_itinerary_request.json`.

### 4: Itinerary response from DynAA to MECA

The result of the evaluation returned by DynAA is also formatted according to the itinerary schema (`interface_schemas/itinerary_schema_v2.json`). 

Multiple route objects can be included in the response that represent successful route evaluations, where each included route is based on a different assignment of charging point usage and charging duration. Unsuccessful evaluations (i.e. where the evaluation constraints are not met) are not included in the response.

For each route, the information regarding route geometry and the individual segments of the route legs is omitted. Instead, each route leg contains itinerary information describing the start and finish time of the route leg. In addition, the state-of-charge (SOC) of the battery is provided at the start and finish of each waypoint, as well as for each route segment.

An example response is provided in this file: `interface_data/meca_dynaa_itinerary_response.json`.

### 5: Ranked itinerary response from MECA to the HMI

After evaluation by DynAA, MECA assigns a score to each itinerary based on certain key performance indicators (KPIs). Based on the score, a ranking is established, dictating the order in which the itineraries are returned to the HMI.

The response to the HMI contains the full itinerary response from DynAA, augmented with the route geometry and segment information available from OSRM.
In addition, the response includes an `explanation` field, which contains the following properties of each itinerary:

- the score assigned by MECA
- the total cost of charging along the route (euros)
- a textual explanation of the score, containing information with respect to the KPIs, such as the total duration of charging, the brand of the charging poles, any relevant facilities available at charging points, etc.

An example response is provided in this file: `interface_data/hmi_meca_response.json`.

The response is formatted according to a slightly expanded version of the `itinerary schema`. This is formalized in the itineraries/explanation schema `interface_schemas/itineraries_schema_v3.json`.
